/*
    This source code contains routines that will convert binary files to
    source that can be compiled with a 65816 assembler.
    BorlandC 3.1 was used to compile this successfully.

    It is a crime to redistribute these routines in a commercial
    venture of any kind without permission or licensing agreement.

    (C)opyright 1994, 1995 Damaged Cybernetics

    If you have any questions or comments, please contact the
    following ppl via the Internet:

        Donald Moore   (MindRape) mind@primenet.com
        Donald Staheli (Royce)    dstaheli@primenet.com

        Web Page: www.primenet.com/~dstaheli

 */
#include <stdio.h>
#include <stdlib.h>
#include "bin2dcb.h"

FILE *source, *dest;

/*
   void error(char *cMsg,int iErrorCode)
   ENT: char *cMsg      - Error msg to display to the user
        int  iErrorCode - Error Code
   RET: <nothing>

   Display a error msg to the user and exit the program.
   we can do dirty tricks here cause Borland C compilers exit
   function cleans up a lot of stuff
 */
void error(char *cMsg,int iErrorCode)
{
    puts(cMsg);
    exit(iErrorCode);
}


/*
   unsigned long FileLen(FILE *source)
   ENT: FILE *source - ptr to our open file
   RET: filesize of *source

   Grab the filesize of a open FILE * and return it
 */
unsigned long FileLen(FILE *source)
{   unsigned long lTemp;

    fseek(source,0,SEEK_END);      /* seek to the end */
    lTemp = ftell(source);         /* grab filesize   */
    rewind(source);                /* rewind source   */

    return(lTemp);
}


/*
   void bin2dcb()
   ENT: <nothing>
   RET: <nothing>

   take a binary file and convert it to DCB statements.

 */
void bin2dcb()
{
             int  i;
             int  iSize;
    unsigned char ucBuffer[BUFFER_SIZE];
    unsigned long ulFSize;
    unsigned long ulFSizeCounter = 0;
    unsigned  int uiCharCounter = 0;

    /* our little nice header */
    fputs(cAdvertiseMsg,dest);
   
    /* usualy we won't go over over 64K */
    ulFSize = FileLen(source);

    /* write out the size of our bin */
    fprintf(dest,"DataDumpSize DC.W $%04X    ; size of our data\n",ulFSize);

    /* write out label to identify our converted bin */
    fprintf(dest,"DataDump:");

    /*
       Now we will loop thru the source N BUFFER_SIZE blocks at a
       time (to help speed up our conversion process).

       As we will put X number of bytes per line (CHARS_PER_LINE) in hex
       format.  Once we reached the max bytes per line, insert a tab,
       and carriage routine and starting DC.B declare statement.
     */
    while (!feof(source))
	{
        iSize = fread(ucBuffer,1,BUFFER_SIZE,source);

	    for (i=0;i<iSize;i++)
		{
		    if (!uiCharCounter)
			fprintf(dest,"\n\x09 DC.B ");   // each line needs to start out with dc.b

		    uiCharCounter = (uiCharCounter++ > CHARS_PER_LINE) ? 0 : uiCharCounter;

            fprintf(dest,"$%02X",ucBuffer[i]);
		    ulFSizeCounter++;

		    if (ulFSizeCounter < ulFSize && i < iSize)
			fputc(',',dest);
		}
	}

    fcloseall();
}


void main(int argc,char *argv[])
{
    puts(cTitleMsg);

    if (argc < 3)
        error("bin2dcb <input file> <output file>",3);

    if ((source = fopen(argv[1],"rb")) == NULL)
        error("Cannot open input file!",1);

    if ((dest   = fopen(argv[2],"wb")) == NULL)
        error("Cannot create output file!",2);

    bin2dcb();
}
